@@ -46,7 +46,7 @@ import char *exec_chdir_to;
 import int db_data_subdirs;
 import char proto_host[];
 #ifdef NNTP
-import char nntp_server[];
+import char *nntp_server;
 #endif
 
 static char *pre_input;
@@ -105,7 +105,7 @@ long min_val, max_val;
 
     tprintf("%s %ld..%ld (or all): ", prompt_str, min_val, max_val);
     fl;
-    gets(buf);
+    fgets(buf,sizeof(buf),stdin);
     if (buf[0] == 'a' || buf[0] == NUL)
 	return -1L;
 
@@ -771,7 +771,7 @@ char *mode_str;
 int set_flag;
 {
     char buffer[50];
-    int new_flag;
+    int new_flag = 0;
 
     tputc(NL);
 
