@@ -6,22 +6,26 @@
 
 #include "config.h"
 #include "db.h"
+#include "nn_term.h"
 
 #ifdef NOV
 #include "hash.h"
 #include "hdbm.h"
 #include "newsoverview.h"
-#endif /* NOV */
+#endif	/* NOV */
 
 /* db.c */
 
 #ifndef NOV
 static char *mk_archive_file __APROTO((register group_header *gh, register char *cp, int logerr));
-#endif
+#endif	/* !NOV */
+
 static sort_gh __APROTO((group_header **g1, group_header **g2));
+struct novgroup *nntp_get_overview(group_header *, article_number, article_number);
 
 #ifdef NOV
 static void db_init_group __APROTO((group_header *gh, int num));
+static void db_init_active __APROTO((group_header *gh));
 static void readtimfile __APROTO((void));
 static void readactfile __APROTO((void));
 static void db_fixup_cross_postings __APROTO((data_header *dhp,
@@ -35,7 +39,7 @@ static HASHTABLE *timtbl;
 
 extern long	atol();
 extern void novclose();		/* trash last group's data */
-#endif
+#endif	/* NOV */
 
 #ifdef	CACHE_PURPOSE
 #include <ctype.h>
@@ -84,7 +88,7 @@ export master_header master;
 export group_header **active_groups = NULL;
 #else
 export group_header *active_groups = NULL;
-#endif
+#endif	/* MALLOC_64K_LIMITATION */
 export group_header **sorted_groups = NULL;
 
 export int  reread_groups_file = 0;  /* nnmaster -G */
@@ -117,10 +121,10 @@ register group_header *gh;
 
     current_digest_article = 0;
 
-    if (gh == NULL) 
+    if (gh == NULL)
       return 0;
     /*    if (gh->master_flag & M_IGNORE_GROUP) return 0; */	/* OBS */
-    if (gh == current_group) 
+    if (gh == current_group)
       return 1;
 
     current_group = gh;
@@ -135,7 +139,7 @@ register group_header *gh;
 #ifdef NNTP
     if (use_nntp && nntp_set_group(gh) < 0)
 	return 0;
-#endif /* NNTP */
+#endif	/* NNTP */
 
     if (group_position == NULL)
 	if (who_am_i == I_AM_MASTER || who_am_i == I_AM_EXPIRE)
@@ -168,12 +172,12 @@ register group_header *gh;
     }
 
     /* client */
-    if (gh->master_flag & M_NO_DIRECTORY) 
+    if (gh->master_flag & M_NO_DIRECTORY)
         return 0;
 
     if (check_group_access && !use_nntp) {
 	*p = NUL;
-	if (file_exist(group_path_name, "dxr") == 0) 
+	if (file_exist(group_path_name, "dxr") == 0)
 	    return 0;
     }
 
@@ -213,14 +217,14 @@ int mode;
     group_file = open_file(relative(db_directory, "GROUPS"), mode);
 
     if (group_file != NULL && (mode & OPEN_CREATE)) {
-	fprintf(group_file, 
+	fprintf(group_file,
 "#\n#\tNEVER MODIFY THIS FILE WHILE nnmaster IS RUNNING\n");
-	fprintf(group_file, 
+	fprintf(group_file,
 "#\n#\tRUN 'nnmaster -G' AFTER MODIFYING THIS FILE\n");
-	fprintf(group_file, 
+	fprintf(group_file,
 "#\n#\tDO NOT REMOVE OR REORDER ANY LINES IN THIS FILE\n#\n");
     }
-    
+
     return group_file != NULL;
 }
 
@@ -283,7 +287,7 @@ group_header *group_list;
     if (save_groups)
 	if (save_old_file(relative(db_directory, "GROUPS"), "~") < 0)
 	    sys_error("Cannot rename GROUPS file");
-    
+
     open_groups(OPEN_CREATE|MUST_EXIST);
 
     if (group_list != NULL) {
@@ -304,13 +308,13 @@ int logerr;
 {
     char *name;
 
-    while (*cp && (*cp == '>' || isspace(*cp))) 
+    while (*cp && (*cp == '>' || isspace(*cp)))
         cp++;
 
     name = cp;
-    while (*cp && !isspace(*cp)) 
+    while (*cp && !isspace(*cp))
         cp++;
-    if (*cp) 
+    if (*cp)
         *cp++ = NUL;
 
     if (*name) {
@@ -474,7 +478,7 @@ int trust_master;		/* trust what is in the master file */
 	}
 	gh->master_flag &= ~M_IGNORE_G;
     }
-    
+
     return 1;
 }
 
@@ -490,7 +494,7 @@ int force_copy;
   char client_path[FILENAME];
   static int pass_count = 0;
   int n;
-    
+
   if (!force_copy) {	/* if not forced copy, only do copy every %100 passes */
       pass_count++;
       if (pass_count < 100) {
@@ -498,14 +502,14 @@ int force_copy;
       }
   }
   pass_count = 0;
-    
+
   strcpy(client_path,relative(db_directory, "CLIENT"));
   unlink(client_path);
   if ((n = copy_file(relative(db_directory, "MASTER"), client_path, 0)) < 0)
       log_entry('R', "Copy of MASTER to CLIENT failed (err=%d)", n);
 }
-#endif
-#endif /* NOV */
+#endif	/* APOLLO_DOMAIN_OS */
+#endif	/* !NOV */
 
 #ifdef NOV
 /*
@@ -526,11 +530,17 @@ int	mode;
     readtimfile();		/* Read the newsgroup creation time file */
 
     db_expand_master();		/* uses count from readact() call! */
-  
+
     Loop_Groups_Header(gh) {
         /* db_read_group opens a file per call; use db_init_group instead */
         db_init_group(gh, l_g_index);
     }
+  
+    sort_groups();
+
+    Loop_Groups_Header(gh) {
+	db_init_active(gh);
+    }
 
     /*
      * Free actlist and timtbl space.
@@ -542,11 +552,9 @@ int	mode;
     pfree(POOL_TIM);
     free(timtbl);
 
-    sort_groups();
-
 #ifdef	CACHE_PURPOSE
     cache_purpose();	/* cache sorted newsgroups and descriptions */
-#endif
+#endif	/* CACHE_PURPOSE */
 }
 
 #else /* NOV */
@@ -568,7 +576,7 @@ int mode;
     if (who_am_i != I_AM_MASTER && who_am_i != I_AM_ADMIN)
 	master_file = open_file(relative(db_directory, "CLIENT"), mode|MUST_EXIST);
     else
-#endif
+#endif	/* APOLLO_DOMAIN_OS */
     master_file = open_file(relative(db_directory, "MASTER"), mode|MUST_EXIST);
 
     db_sequential = 0;
@@ -585,7 +593,7 @@ int mode;
 #ifdef MALLOC_64K_LIMITATION
     if (active_groups)
 	Loop_Groups_Header(gh) freeobj(gh);
-#endif
+#endif	/* MALLOC_64K_LIMITATION */
     freeobj(active_groups);
     active_groups = NULL;
     sorted_groups = NULL;
@@ -603,7 +611,7 @@ int mode;
 	active_groups[l_g_index] = gh;
 #else
     Loop_Groups_Header(gh) {
-#endif
+#endif	/* MALLOC_64K_LIMITATION */
 	gh->group_num = l_g_index;
 	db_read_group(gh);
 	db_parse_group(gh, trust_master);
@@ -616,9 +624,9 @@ int mode;
 
 #ifdef	CACHE_PURPOSE
     cache_purpose();	/* cache sorted newsgroups and descriptions */
-#endif
+#endif	/* CACHE_PURPOSE */
 }
-#endif /* NOV */
+#endif	/* NOV */
 
 void
 db_expand_master()
@@ -629,18 +637,18 @@ db_expand_master()
 #ifdef MALLOC_64K_LIMITATION
     active_groups = resizeobj(active_groups, group_header *,
 			      master.number_of_groups + master.free_groups);
-#else
+#else	/* MALLOC_64K_LIMITATION */
     active_groups = resizeobj(active_groups, group_header,
 			      master.number_of_groups + master.free_groups);
     clearobj(active_groups + master.number_of_groups, group_header,
 	     master.free_groups);
-#endif /* MALLOC_64K_LIMITATION */
-#else
+#endif	/* MALLOC_64K_LIMITATION */
+#else	/* !NOV */
     active_groups = resizeobj(active_groups, group_header,
 			      master.number_of_groups + master.free_groups);
     clearobj(active_groups + master.number_of_groups, group_header,
 	     master.free_groups);
-#endif /* NOV */
+#endif	/* !NOV */
     sorted_groups = resizeobj(sorted_groups, group_header * , master.number_of_groups + master.free_groups);
 }
 
@@ -654,8 +662,8 @@ char *key, *data, *hook;
     free(key);
     free(data);
 }
-#endif /* NOV */
-#endif /* 0 */
+#endif	/* NOV */
+#endif	/* 0 */
 
 void
 close_master()
@@ -666,7 +674,7 @@ close_master()
 	novclose(ngovp);
 	ngovp = NULL;
     }
-#endif
+#endif	/* 0 */
 	
 #else /* NOV */
     if (master_file != NULL) {
@@ -675,9 +683,9 @@ close_master()
 
 #ifdef APOLLO_DOMAIN_OS
 	if (who_am_i == I_AM_MASTER) make_master_copy(1);
-#endif
+#endif	/* APOLLO_DOMAIN_OS */
     }
-#endif /* NOV */
+#endif	/* NOV */
 }
 
 int
@@ -692,13 +700,13 @@ group_header *gh;
     db_read_master();
     master.number_of_groups = numg;
     if (master.db_lock[0]) return -3;
-#endif /* NOV */
+#endif	/* !NOV */
 
 #ifndef NOV
     db_read_group(gh);
 #else
     db_read_group(gh, gh->first_a_article, gh->last_a_article);	/* XXX */
-#endif
+#endif	/* !NOV */
 
     if (gh->master_flag & M_IGNORE_GROUP) return 0;
     if (gh->master_flag & M_BLOCKED) return -1;
@@ -809,11 +817,11 @@ char d_or_x;
 	sprintf(namebuf, "%s/%s.%c", db_data_directory, gh->group_name, d_or_x);
 #else
 	if (db_data_subdirs)
-	    sprintf(namebuf, "%s/%d/%d.%c", db_data_directory, 
+	    sprintf(namebuf, "%s/%ld/%ld.%c", db_data_directory,
 		    gh->group_num/100, gh->group_num, d_or_x);
 	else
-	sprintf(namebuf, "%s/%d.%c", db_data_directory, gh->group_num, d_or_x);
-#endif
+	sprintf(namebuf, "%s/%ld.%c", db_data_directory, gh->group_num, d_or_x);
+#endif	/* DB_LONG_NAMES */
     } else {
 	np = namebuf;
 	/* master chdir to the group's directory */
@@ -869,7 +877,7 @@ int mode;
 	    *s = '/';
 	    errno = ENOENT;
 	}
-#endif
+#endif	/* DB_LONG_NAMES */
 	if (mode & MUST_EXIST)
 	    sys_error("%s (%d): cannot open '%c' file (mode=%x, errno=%d)",
 		      gh->group_name, (int)(gh->group_num), d_or_x,
@@ -915,8 +923,8 @@ int lgt;
     }
 }
 
-#endif /* not NETWORK_BYTE_ORDER */
-#endif /* NETWORK_DATABASE */
+#endif	/* not NETWORK_BYTE_ORDER */
+#endif	/* NETWORK_DATABASE */
 
 #define NWDB_MAGIC 0x00190000	/* NN#n <-> NW#n */
 
@@ -944,7 +952,7 @@ db_read_master()
     rewind(master_file);
     if (fread((char *)&master, sizeof(master_header), 1, master_file) != 1)
 	goto err;
-#endif
+#endif	/* NETWORK_DATABASE */
 
     if (master.db_magic != NNDB_MAGIC)
 	sys_error("Database magic number mismatch");
@@ -972,11 +980,11 @@ readactfile()
     if (actlist != NULL)
 	return;
 
-#ifdef NNTP 
+#ifdef NNTP
     if (use_nntp) {
 	actfp = nntp_fopen_list("LIST");
     } else
-#endif
+#endif	/* NNTP */
 	actfp = fopen(relative(news_lib_directory, "active"), "r");
 
     if (actfp == NULL) {
@@ -993,7 +1001,7 @@ readactfile()
 		      :  fgets(actline, sizeof actline, actfp) )
 #else
     while ( fgets(actline, sizeof actline, actfp) )
-#endif
+#endif	/* NNTP */
     {
 	stlist_t *stnew = (stlist_t *)strkeep(actline,sizeof(stlisthdr_t),POOL_ACT);
 	if (stnew == NULL) {
@@ -1045,7 +1053,7 @@ readactfile()
 
 }
 
-#endif /* NOV */
+#endif	/* NOV */
 
 
 
@@ -1073,12 +1081,12 @@ db_write_master()
     rewind(master_file);
     if (fwrite((char *)&master, sizeof(master_header), 1, master_file) != 1)
 	goto err;
-#endif
+#endif	/* NETWORK_DATABASE */
 
     fflush(master_file);
 #ifdef APOLLO_DOMAIN_OS
     if (who_am_i == I_AM_MASTER) make_master_copy(0);
-#endif
+#endif	/* APOLLO_DOMAIN_OS */
     return;
 
  err:
@@ -1114,12 +1122,12 @@ register group_header *gh;
     gh->creation_time = buf[8];
 #else
     if (!db_sequential)
-	fseek(master_file, 
+	fseek(master_file,
 	      (off_t)(sizeof(master_header) + SAVED_GROUP_HEADER_SIZE(*gh) * gh->group_num), 0);
 
     if (fread((char *)gh, SAVED_GROUP_HEADER_SIZE(*gh), 1, master_file) != 1)
 	goto err;
-#endif
+#endif	/* NETWORK_DATABASE */
 
     return;
 
@@ -1153,26 +1161,26 @@ register group_header *gh;
     host_to_net(buf, GROUP_FIELDS);
     if (fwrite((char *)buf, sizeof(net_long), GROUP_FIELDS, master_file) != GROUP_FIELDS)
 	goto err;
-#else
+#else	/* NETWORK_DATABASE */
     if (!db_sequential)
 	fseek(master_file, (off_t)(sizeof(master_header) + SAVED_GROUP_HEADER_SIZE(*gh) * gh->group_num), 0);
 
 
     if (fwrite((char *)gh, SAVED_GROUP_HEADER_SIZE(*gh), 1, master_file) != 1)
 	goto err;
-#endif
+#endif	/* NETWORK_DATABASE */
 
     fflush(master_file);
 
 #ifdef APOLLO_DOMAIN_OS
     if (who_am_i == I_AM_MASTER) make_master_copy(0);
-#endif
+#endif	/* APOLLO_DOMAIN_OS */
     return;
 
  err:
     sys_error("Write GROUPS failed");
 }
-#endif
+#endif	/* !NOV */
 
 
 #ifdef NOV
@@ -1181,7 +1189,7 @@ readtimfile()
 {
     char timline[512];
     FILE *timfp;
-    int hsize;
+    unsigned hsize;
 
     if (timtbl != NULL)
 	return;
@@ -1195,7 +1203,7 @@ readtimfile()
     if (use_nntp)
 	timfp = nntp_fopen_list("LIST active.times");
     else
-#endif
+#endif	/* NNTP */
 	timfp = fopen(relative(news_lib_directory, "active.times"), "r");
 
     if (timfp == NULL)
@@ -1206,9 +1214,9 @@ readtimfile()
 #ifdef NNTP
     while ( use_nntp  ?  nntp_fgets(timline, sizeof timline)
 		      :  fgets(timline, sizeof timline, timfp) )
-#else
+#else	/* NNTP */
     while ( fgets(timline, sizeof timline, timfp) )
-#endif
+#endif	/* NNTP */
     {
 	char *line = strkeep(timline, 0, POOL_TIM);
 	char *p = strchr(line, ' ');
@@ -1225,7 +1233,7 @@ readtimfile()
 	(void) fclose(timfp);
 
 }
-#endif /* NOV */
+#endif	/* NOV */
 
 #ifdef	CACHE_PURPOSE
 static int
@@ -1364,10 +1372,10 @@ FILE *f;
     db_hdr.dh_cross_postings = buf[7];
     db_hdr.dh_subject_length = buf[8];
     db_hdr.dh_sender_length = buf[9];
-#else
+#else	/* NETWORK_DATABASE */
     if (fread((char *)&db_hdr, sizeof(data_header), 1, f) != 1) return 0;
     bytes = sizeof(data_header);
-#endif
+#endif	/* NETWORK_DATABASE */
 
     if (db_hdr.dh_number < 0) {
 	current_digest_article = db_hdr.dh_number = -db_hdr.dh_number;
@@ -1408,7 +1416,7 @@ FILE *f;
 
     return bytes;
 }
-#endif /* NOV */
+#endif	/* !NOV */
 
 
 #ifdef NOV
@@ -1428,7 +1436,7 @@ int	num;
 	gh->group_num = num;
 	if (gh->group_name == NULL) {
 	    gh->group_name = grplist[num];
-	    if (gh->group_name == NULL) {  
+	    if (gh->group_name == NULL) {
 		nn_exitmsg(1, "can't map group %d to name\n", num);
 	    }
 	    gh->group_name_length = strlen(gh->group_name);
@@ -1451,20 +1459,43 @@ int	num;
 	if (p != NULL) {
 	    gh->creation_time = atol(p);
 	}
+}
 
-	p = actlist[num];
+/*
+ * further initialise *gh; to be done after sort_groups()
+ */
+void
+db_init_active(gh)
+register group_header *gh;
+{
+	group_header	*gh1;
+	register char	*p;
+
+	p = actlist[gh->group_num];
 	if (p != NULL) {
-	    gh->last_a_article = atol(++p);
+	    while (isspace(*p)) ++p;
+	    gh->last_a_article = atol(p);
 	    p = strchr(p, ' ');
 	    if (p == NULL)
 		return;
 	    gh->first_a_article = atol(++p);
+	    p = strchr(p, ' ');
+	    if (*++p == '=') {			/* an alias */
+		gh1 = lookup_no_alias(++p);
+		if (gh1 == NULL) {
+		    log_entry('R', "Group %s aliased to unknown group (%s)",
+			      gh->group_name, p);
+			      /* ah well! leave it be */
+		} else {
+		    gh->master_flag |= M_ALIASED;
+		    gh->data_write_offset = (off_t)gh1->group_num;
+		}
+	    }
 	}
 	gh->first_db_article = gh->first_a_article; /* lowest # in ov. data */
 	gh->last_db_article = gh->last_a_article; /* highest # in ov. data */
 }
 
-
 /*
  * slurp up the overview data for this group into *gh.
  * this costs a file open and so should not be done frivolously.
@@ -1485,7 +1516,7 @@ article_number first, last;
 	if (use_nntp) {
 		ngovp = nntp_get_overview(gh, first, last);
 	} else
-#endif
+#endif	/* NNTP */
 		ngovp = novopen(gh->group_name);
 
 	if (ngovp == NULL) {
@@ -1507,7 +1538,7 @@ article_number first, last;
 
 		lastartp = allarts;
 		for (artp = allarts; artp != NULL; artp = artp->a_nxtnum)
-			if (artp->a_num[0] != '0' || artp->a_num[1] != 0)
+			if (atol(artp->a_num) != 0)
 			    lastartp = artp;
 
 		gh->last_db_article  = atol(lastartp->a_num); /* highest # */
@@ -1577,7 +1608,7 @@ FILE *f;
 				}
 			}
 		}
-#endif
+#endif	/* DO_NOV_DIGEST */
 		dhp->dh_number = current_digest_article;
 		ddp->dh_type = DH_SUB_DIGEST;
 	} else {
@@ -1604,7 +1635,7 @@ struct novart *artp;
     char *curg, *tmp;
     int numgrps = 0;
 
-    dhp->dh_cross_postings = 0; /* assume none as default until we can show 
+    dhp->dh_cross_postings = 0; /* assume none as default until we can show
 				   otherwise */
 
     /* If no "other" header lines are in NOV database, we're out of luck,
@@ -1658,13 +1689,13 @@ struct novart *artp;
 	while (isdigit(*curg)) ++curg; /* Skip over the article number */
     }
     if (numgrps > 1) {
-	/* Note: if # of groups is only 1, we leave dh_cross_postings 
+	/* Note: if # of groups is only 1, we leave dh_cross_postings
 	   at its original value of zero. */
 	dhp->dh_cross_postings = numgrps;
     }
     return;
 }
-#endif /* NOV */
+#endif	/* NOV */
 
 
 #ifndef NOV
@@ -1674,7 +1705,7 @@ FILE *f;
 {
 #ifdef NETWORK_DATABASE
     net_long buf[ARTICLE_FIELDS];
-#endif
+#endif	/* NETWORK_DATABASE */
     article_number art_num = db_hdr.dh_number;
 
     switch (db_data.dh_type) {
@@ -1704,12 +1735,12 @@ FILE *f;
 
     if (fwrite((char *)buf, sizeof(net_long), ARTICLE_FIELDS, f) != ARTICLE_FIELDS)
 	return -1;
-#else
+#else	/* NETWORK_DATABASE */
 
     if (fwrite((char *)&db_hdr, sizeof(data_header), 1, f) != 1)
 	return -1;
 
-#endif
+#endif	/* NETWORK_DATABASE */
     if (db_hdr.dh_cross_postings)
     	if (fwrite((char *)db_data.dh_cross, sizeof(cross_post_number),
 		  (int)db_hdr.dh_cross_postings, f)
@@ -1739,9 +1770,9 @@ article_number art_num;
 {
 #ifdef NETWORK_DATABASE
     return (off_t)((art_num - gh->first_db_article) * sizeof(net_long));
-#else
+#else	/* NETWORK_DATABASE */
     return (off_t)((art_num - gh->first_db_article) * sizeof(off_t));
-#endif
+#endif	/* NETWORK_DATABASE */
 }
 
 off_t get_data_offset(gh, art_num)
@@ -1778,12 +1809,12 @@ off_t *offset;
 
 #ifndef NETWORK_BYTE_ORDER
     temp = ntohl(temp);
-#endif
+#endif	/* !NETWORK_BYTE_ORDER */
     *offset = temp;
-#else
+#else	/* NETWORK_DATABASE */
 
     if (fread((char *)offset, sizeof(off_t), 1, f) != 1) return 0;
-#endif
+#endif	/* NETWORK_DATABASE */
     return 1;
 }
 
@@ -1799,17 +1830,17 @@ off_t *offset;
 
 #ifndef NETWORK_BYTE_ORDER
     temp = htonl(temp);
-#endif
+#endif	/* !NETWORK_BYTE_ORDER */
     if (fwrite((char *)&temp, sizeof(net_long), 1, f) != 1) return 0;
 
-#else
+#else	/* NETWORK_DATABASE */
 
     if (fwrite((char *)offset, sizeof(off_t), 1, f) != 1) return 0;
-#endif
+#endif	/* NETWORK_DATABASE */
     return 1;
 }
 
-#endif /* NOV */
+#endif	/* !NOV */
 
 
 #ifdef NOV
@@ -1860,7 +1891,7 @@ article_number art_num;
   nn_exitmsg(50, "STUB ROUTINE CALLED: get_index_offset\n");
   return -1;
 }
-#endif /* NOV */
+#endif	/* NOV */
 
 #if defined(NOV) || defined(CACHE_PURPOSE)
 /*
@@ -1972,8 +2003,6 @@ int poolid;
 {
     register int size;
     register char *ret;
-    register stpool_t *pp;
-    register stlist_t *stnew;
 
     if (poolid == POOL_TMP) {
 	size = hdr + strlen(s);
@@ -1989,7 +2018,7 @@ int poolid;
 	bcopy(s, ret+hdr, size-hdr);
 #else
 	memmove(ret+hdr, s, size-hdr);
-#endif /* NO_MEMMOVE */
+#endif	/* NO_MEMMOVE */
 
     return ret;
 }
