@@ -18,6 +18,8 @@ static void more_auto_select __APROTO((void));
 static char *a_st_flags __APROTO((flag_type flag));
 static void brief_header_line __APROTO((register article_header *ah, int lno));
 
+extern void xterm_mouse_on();
+extern void xterm_mouse_off();
 
 export int  monitor_mode = 0;
 export int  compress_mode = 0;
@@ -25,7 +27,11 @@ export int  show_article_date = 1;
 export int  first_page_lines = 0;
 export int  overlap = 2;
 export int  mark_overlap = 0;
+#ifdef GNKSA
+export char *header_lines = "=F_S_D_n_W_R";
+#else
 export char *header_lines = NULL;
+#endif
 export int  min_pv_window = 7;
 export int  wrap_headers = 6;
 export int  data_bits = 7;
@@ -1083,6 +1089,7 @@ no_print:
 		STATE(K_CONTINUE);
 	     }
 	 }
+	/* fall through??? */
 
      case K_NEXT_PAGE:
 	if (eof) {
@@ -1149,15 +1156,18 @@ no_print:
 
 	 case AC_QUIT:
 	    more_return( MC_QUIT );
+	    break;	/* for lint */
 
 	 case AC_PROMPT:
 	    goto dflt_prompt;
 
 	 case AC_REENTER_GROUP:
 	    more_return( MC_REENTER_GROUP );
+	    break;	/* for lint */
 
 	 case AC_REORDER:
 	    more_return( MC_MENU );
+	    break;	/* for lint */
 
 	 case AC_REDRAW:
 	    goto redraw;
@@ -1170,10 +1180,12 @@ no_print:
      case K_QUIT:
 	ah->attr = A_LEAVE_NEXT;
 	more_return( MC_QUIT );
+	break;	/* for lint */
 
      case K_SAVE_NO_HEADER:
      case K_SAVE_SHORT_HEADER:
      case K_SAVE_FULL_HEADER:
+     case K_SAVE_HEADER_ONLY:
      case K_PRINT:
      case K_UNSHAR:
      case K_PATCH:
@@ -1199,9 +1211,6 @@ no_print:
 	goto Prompt;
 
      case K_FOLLOW_UP:
-#ifdef NNTP_POST
-	 if (use_nntp && nntp_no_post()) goto Prompt;
-#endif
      case K_REPLY:
      case K_MAIL_OR_FORWARD:
 	news_save = news;
@@ -1228,11 +1237,13 @@ no_print:
 
 	if (cancel(ah) > 0) goto Prompt;
 	more_return(MC_NEXT);
+	break;	/* for lint */
 
      case K_UNSUBSCRIBE:
 	if (!unsubscribe(current_group)) goto Prompt;
 	if ((current_group->group_flag & G_UNSUBSCRIBED) == 0) goto Prompt;
 	more_return(MC_NEXTGROUP);
+	break;	/* for lint */
 
      case K_GROUP_OVERVIEW:
 	group_overview(-1);
@@ -1256,13 +1267,16 @@ no_print:
 	prompt("Mark rest of current group as read?");
 	if (yes(1) <= 0) goto Prompt;
 	more_return(MC_READGROUP);
+	break;	/* for lint */
 
      case K_NEXT_GROUP_NO_UPDATE:
 	if (mode & MM_PREVIEW) more_return(MC_MENU);
 	more_return(MC_NEXTGROUP);
+	break;	/* for lint */
 
      case K_BACK_TO_MENU:
 	more_return(MC_MENU);
+	break;	/* for lint */
 
      case K_PREVIOUS:
 	if ((mode & MM_PREVIOUS) == 0) {
@@ -1383,6 +1397,7 @@ no_print:
 
      case K_SELECT_SUBJECT:
 	more_return(MC_ALLSUBJ);
+	break;	/* for lint */
 
      case K_HEADER_PAGE:
 	fseek(art, linepos[0], 0);
@@ -1447,6 +1462,7 @@ no_print:
 
 	 case MC_QUIT:
 	    more_return( MC_QUIT );
+	    break;	/* for lint */
 
 	 case MC_REENTER_GROUP:
 	    more_return( MC_REENTER_GROUP );
@@ -1458,6 +1474,7 @@ no_print:
      case K_LEAVE_NEXT:
 	ah->attr = A_LEAVE_NEXT;
 	more_return(MC_PREVIEW_NEXT);
+	break;	/* for lint */
 
      case K_LEAVE_ARTICLE:
 	ah->attr = (mode & MM_PREVIEW) ? A_SELECT : A_LEAVE;
@@ -1466,6 +1483,7 @@ no_print:
      case K_NEXT_ARTICLE:
 	if ((mode & MM_PREVIEW) == 0) break;
 	more_return(MC_PREVIEW_NEXT);
+	break;	/* for lint */
 
      case K_BACK_ARTICLE:
 	if (mode & MM_FIRST_ARTICLE) {
@@ -1473,6 +1491,7 @@ no_print:
 	    goto same_prompt;
 	}
 	more_return(MC_BACK_ART);
+	break;	/* for lint */
 
      case K_FORW_ARTICLE:
 	if (mode & MM_LAST_ARTICLE) {
@@ -1480,9 +1499,11 @@ no_print:
 	    goto same_prompt;
 	}
 	more_return(MC_FORW_ART);
+	break;	/* for lint */
 
      case K_NEXT_SUBJECT:
 	more_return(MC_NEXTSUBJ);
+	break;	/* for lint */
 
      case K_ROT13:
 	if (rot13_must_init) {
